/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * BinarySlamMap.h
 *
 *  Created on: Jul 31, 2009
 *      Author: jwienke
 */

#ifndef BINARYSLAMMAP_H_
#define BINARYSLAMMAP_H_

#include "DynamicGridMap.h"
#include "PositionData.h"

#include <xmltio/xmltio.hpp>

// Smart Pointer
#include <boost/shared_ptr.hpp>

namespace airobots {

/**
 * This is a container for result data from a slam component. This includes a
 * grid map representing the binary data, meta information like resolution and
 * origin of the map and information about the robots current position.
 *
 * @author jwienke, lziegler
 */
class BinarySlamMap: public PositionData {

public:

	static std::string getBaseTag();

	static std::string getTemplate();

	BinarySlamMap();
	BinarySlamMap(std::string generator);
	BinarySlamMap(const BinarySlamMap &map);
	BinarySlamMap& operator=(BinarySlamMap const& map);
	virtual ~BinarySlamMap();

	/**
	 * Returns the attachment uri of the binary slam map.
	 */
	std::string getUri() const;
	void setUri(std::string uri);

	/**
	 * Returns the x size of the map in map units.
	 */
	unsigned int getSizeX() const;
	void setSizeX(unsigned int sizeX);

	/**
	 * Returns the y size of the map in map units.
	 */
	unsigned int getSizeY() const;
	void setSizeY(unsigned int sizeY);

	/**
	 * Returns the x origin of the map in map units.
	 */
	unsigned int getOriginX() const;
	void setOriginX(unsigned int originX);

	/**
	 * Returns the y origin of the map in map units.
	 */
	unsigned int getOriginY() const;
	void setOriginY(unsigned int originY);

	/**
	 * Returns the resolution of the map. That is:
	 * map units * resolution = meters.
	 */
	float getResolution() const;
	void setResolution(float resolution);

	/**
	 * Returns the map itself.
	 */
	const GridMap& getGridMap() const;
	void setGridMap(GridMap &gridMap);
	void setGridMap(const std::vector<unsigned char> &charArray,
			unsigned int width, unsigned int height);
	void setGridMap(const std::vector<float> &floatArray, unsigned int width,
			unsigned int heigth);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, BinarySlamMap &type);

private:

	DynamicGridMap dynamicGridMap;

};

/**
 * Smart pointer for this class
 */
typedef boost::shared_ptr<BinarySlamMap> BinarySlamMapPtr;

}

#endif /* BINARYSLAMMAP_H_ */
