/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * Rotation3DTest.cpp
 *
 *  Created on: Apr 11, 2011
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/Rotation3D.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(Rotation3DTest)


BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {
		double m[9] = {3.0, 45.0, 2.0, 0.5, 0.1, 2.9, 3.89, 0.4, 23.9};
		Eigen::Matrix3d mat(m);
		Rotation3D original(mat);

		Location originalLoc(Rotation3D::getTemplate(), Rotation3D::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		Rotation3D parsed = extract<Rotation3D>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getMatrix(), original.getMatrix());

	} catch (ParsingException &e) {
		BOOST_FAIL(string("Caught parsing exception: ") + e.what());
	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());

	}

	try {
		Eigen::Vector3d axis(3.0,6.0,5.0);
		Rotation3D original(axis, 0.5, RADIAN);

		axis.normalize();

		Eigen::Vector3d resultAxis0;
		double resultAngle0;
		original.getAxisAngle(resultAxis0,resultAngle0,RADIAN);

		BOOST_REQUIRE(resultAxis0.isApprox(axis));
		BOOST_REQUIRE_CLOSE(resultAngle0, 0.5, 0.00001);

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

	try {
		Eigen::Quaterniond q(0.7,0.1,0.4,0.0);
		Rotation3D original(q);

		Location originalLoc(Rotation3D::getTemplate(), Rotation3D::getBaseTag());
		originalLoc = original;
		cout << originalLoc.getDocumentText(true) << endl;

		Eigen::Quaterniond qNew = original.getQuaternion();
		q.normalize();

		BOOST_REQUIRE_EQUAL(q.coeffs(), qNew.coeffs());

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

	try {
		Eigen::Vector3d axis(3.0,6.0,5.0);
		Rotation3D original(axis, 0.5, RADIAN);
		Eigen::Quaterniond q = original.getQuaternion();
		Rotation3D tmp(q);

		Location tmpLoc(Rotation3D::getTemplate(), Rotation3D::getBaseTag());
		tmpLoc = tmp;
		cout << tmpLoc.getDocumentText(true) << endl;

		Eigen::Vector3d resultAxis0;
		double resultAngle0;
		original.getAxisAngle(resultAxis0,resultAngle0,RADIAN);

		axis.normalize();
		BOOST_REQUIRE(resultAxis0.isApprox(axis));
		BOOST_REQUIRE_CLOSE(resultAngle0, 0.5, 0.00001);

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

	try {
		Eigen::Vector3d euler(M_PI_2,0.2,0.1);

		Rotation3D original(euler, RADIAN);

		Location originalLoc(Rotation3D::getTemplate(), Rotation3D::getBaseTag());
		originalLoc = original;
		cout << originalLoc.getDocumentText(true) << endl;

		Eigen::Vector3d eulerNew = original.getEuler(RADIAN);

		BOOST_REQUIRE(euler.isApprox(eulerNew));

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}
}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Rotation3D.xml"), Rotation3D::getBaseTag());
	Rotation3D parsed = extract<Rotation3D>(fileContents);

	Eigen::Matrix3d mat = parsed.getMatrix();
	BOOST_REQUIRE_CLOSE(0, mat(0,0), TOLERANCE);
	BOOST_REQUIRE_CLOSE(1, mat(0,1), TOLERANCE);
	BOOST_REQUIRE_CLOSE(2, mat(0,2), TOLERANCE);
	BOOST_REQUIRE_CLOSE(3, mat(1,0), TOLERANCE);
	BOOST_REQUIRE_CLOSE(4, mat(1,1), TOLERANCE);
	BOOST_REQUIRE_CLOSE(5, mat(1,2), TOLERANCE);
	BOOST_REQUIRE_CLOSE(6, mat(2,0), TOLERANCE);
	BOOST_REQUIRE_CLOSE(7, mat(2,1), TOLERANCE);
	BOOST_REQUIRE_CLOSE(8, mat(2,2), TOLERANCE);
}

BOOST_AUTO_TEST_SUITE_END()
