/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.common;

import javax.vecmath.AxisAngle4d;
import javax.vecmath.Matrix3d;
import javax.vecmath.Quat4d;
import javax.vecmath.Vector3d;

import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.data.Type;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * This is a generic rotation type. It can be set and read using several
 * representation formats.
 * 
 * @author lziegler
 */
public class Rotation3D extends Type {

	private static final String BASE_TAG = "ROTATION3D";
	private static final String MATRIX_TAG = "MATRIX";
	private static final String VALUE_TAG = "VALUE";

	private Matrix3d mat = new Matrix3d();

	/**
	 * Creates a default rotation object (no rotation).
	 */
	public Rotation3D() {
		mat.setIdentity();
	}

	/**
	 * Create rotation object from axis/angle representation.
	 * 
	 * @param x
	 *            First component of the axis.
	 * @param y
	 *            Second component of the axis.
	 * @param z
	 *            Third component of the axis.
	 * @param angle
	 *            Rotation angle around the defined axis.
	 * @param angleUnit
	 *            Unit of the defined angle.
	 */
	public Rotation3D(double x, double y, double z, double angle,
			AngleUnit angleUnit) {

		AxisAngle4d rot = new AxisAngle4d(x, y, z, UnitConverter.convert(angle,
				angleUnit, AngleUnit.RADIAN));
		mat.set(rot);
	}

	/**
	 * Create rotation object from axis/angle representation.
	 * 
	 * @param axis
	 *            The axis as vecmath object.
	 * @param angle
	 *            Rotation angle around the defined axis.
	 * @param angleUnit
	 *            Unit of the defined angle.
	 * @see #Rotation3D(double, double, double, double, AngleUnit)
	 */
	public Rotation3D(Vector3d axis, double angle, AngleUnit angleUnit) {

		AxisAngle4d rot = new AxisAngle4d(axis, UnitConverter.convert(angle,
				angleUnit, AngleUnit.RADIAN));
		mat.set(rot);
	}

	/**
	 * Create rotation object from axis/angle representation.
	 * 
	 * @param rotation
	 *            The rotation as vecmath object.
	 */
	public Rotation3D(AxisAngle4d rotation) {
		mat.set(rotation);
	}

	/**
	 * Create rotation object from quaternion representation.
	 * 
	 * @param rotation
	 *            The rotation as vecmath object.
	 */
	public Rotation3D(Quat4d rotation) {
		mat.set(rotation);
	}

	/**
	 * Create rotation object from euler angles.
	 * 
	 * @param heading
	 *            first rotation (azimuth, yaw, theta)
	 * @param attitude
	 *            second rotation (elevation, pitch, phi)
	 * @param bank
	 *            third rotation (tilt, psi, roll)
	 * @param angleUnit
	 *            Unit of the rotations
	 */
	public Rotation3D(double heading, double attitude, double bank,
			AngleUnit angleUnit) {

		setEuler(heading, attitude, bank, angleUnit);
	}

	/**
	 * Create rotation object from euler angles.
	 * 
	 * @param vec
	 *            A vector containing the angles for heading, attitude and bank.
	 * @param angleUnit
	 *            Unit of the rotations
	 * @see #Rotation3D(double, double, double, AngleUnit)
	 */
	public Rotation3D(Vector3d vec, AngleUnit angleUnit) {
		this(vec.getX(), vec.getY(), vec.getZ(), angleUnit);
	}

	/**
	 * Create rotation object from rotation matrix.
	 * 
	 * @param mat
	 *            The rotation matrix.
	 */
	public Rotation3D(Matrix3d mat) {
		this.mat = mat;
	}

	/**
	 * Returns the rotation in matrix representation.
	 * 
	 * @return The rotation matrix.
	 */
	public Matrix3d getMatrix() {
		return mat;
	}

	/**
	 * Returns the rotation in quaternion representation.
	 * 
	 * @return A vecmath object containing the rotation values.
	 */
	public Quat4d getQuaternion() {
		Quat4d q = new Quat4d();
		q.set(mat);
		return q;
	}

	/**
	 * Returns the rotation in axis/angle representation.
	 * 
	 * @param unit
	 *            The desired unit of the rotaion angle.
	 * @return A vecmath object representing the rotation values.
	 */
	public AxisAngle4d getAxisAngle(AngleUnit unit) {
		AxisAngle4d a = new AxisAngle4d();
		a.set(mat);
		a.setAngle(UnitConverter.convert(a.getAngle(), AngleUnit.RADIAN, unit));
		return a;
	}

	/**
	 * Returns the rotation in euler angles representation.
	 * 
	 * @param unit
	 *            The desired unit of the angles.
	 * @return A vector object containing the rotation angles. Where: <br/>
	 *         first value = azimuth, yaw, theta<br/>
	 *         second value = elevation, pitch, phi<br/>
	 *         third value = tilt, psi, roll
	 */
	public Vector3d getEuler(AngleUnit unit) {

		// Algorithm from here:
		// http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToEuler/index.htm

		double heading, attitude, bank;

		if (mat.m10 > 0.99999) {
			// singularity at north pole
			heading = Math.atan2(mat.m02, mat.m22);
			attitude = Math.PI / 2;
			bank = 0;
		} else if (mat.m10 < -0.99999) {
			// singularity at south pole
			heading = Math.atan2(mat.m02, mat.m22);
			attitude = -Math.PI / 2;
			bank = 0;
		} else {
			heading = Math.atan2(-mat.m20, mat.m00);
			bank = Math.atan2(-mat.m12, mat.m11);
			attitude = Math.asin(mat.m10);
		}
		
		heading = UnitConverter.convert(heading, AngleUnit.RADIAN, unit);
		bank = UnitConverter.convert(bank, AngleUnit.RADIAN, unit);
		attitude = UnitConverter.convert(attitude, AngleUnit.RADIAN, unit);

		return new Vector3d(heading, attitude, bank);
	}

	/**
	 * Set rotation from axis/angle representation.
	 * 
	 * @param x
	 *            First component of the axis.
	 * @param y
	 *            Second component of the axis.
	 * @param z
	 *            Third component of the axis.
	 * @param angle
	 *            Rotation angle around the defined axis.
	 * @param angleUnit
	 *            Unit of the defined angle.
	 */
	public void setAxisAngle(double x, double y, double z, double angle,
			AngleUnit angleUnit) {

		AxisAngle4d rot = new AxisAngle4d(x, y, z, UnitConverter.convert(angle,
				angleUnit, AngleUnit.RADIAN));
		mat.set(rot);
	}

	/**
	 * Set rotation from axis/angle representation.
	 * 
	 * @param axis
	 *            The axis as vecmath object.
	 * @param angle
	 *            Rotation angle around the defined axis.
	 * @param angleUnit
	 *            Unit of the defined angle.
	 * @see #Rotation3D(double, double, double, double, AngleUnit)
	 */
	public void setAxisAngle(Vector3d axis, double angle, AngleUnit angleUnit) {

		AxisAngle4d rot = new AxisAngle4d(axis, UnitConverter.convert(angle,
				angleUnit, AngleUnit.RADIAN));
		mat.set(rot);
	}

	/**
	 * Set rotation from axis/angle representation.
	 * 
	 * @param rotation
	 *            The rotation as vecmath object.
	 */
	public void setAxisAngle(AxisAngle4d rotation) {
		mat.set(rotation);
	}

	/**
	 * Set rotation from quaternion representation.
	 * 
	 * @param rotation
	 *            The rotation as vecmath object.
	 */
	public void setQuaternion(Quat4d rotation) {
		mat.set(rotation);
	}

	/**
	 * Set rotation from euler angles.
	 * 
	 * @param heading
	 *            first rotation (azimuth, yaw, theta)
	 * @param attitude
	 *            second rotation (elevation, pitch, phi)
	 * @param bank
	 *            third rotation (tilt, psi, roll)
	 * @param angleUnit
	 *            Unit of the rotations
	 */
	public void setEuler(double heading, double attitude, double bank,
			AngleUnit angleUnit) {

		// Algorithm from here:
		// http://www.euclideanspace.com/maths/geometry/rotations/conversions/eulerToMatrix/index.htm

		double h = UnitConverter.convert(heading, angleUnit, AngleUnit.RADIAN);
		double a = UnitConverter.convert(attitude, angleUnit, AngleUnit.RADIAN);
		double b = UnitConverter.convert(bank, angleUnit, AngleUnit.RADIAN);

		double ch = Math.cos(h);
		double sh = Math.sin(h);
		double ca = Math.cos(a);
		double sa = Math.sin(a);
		double cb = Math.cos(b);
		double sb = Math.sin(b);

		double[] values = new double[9];
		values[0] = ch * ca;
		values[1] = sh * sb - ch * sa * cb;
		values[2] = ch * sa * sb + sh * cb;
		values[3] = sa;
		values[4] = ca * cb;
		values[5] = -ca * sb;
		values[6] = -sh * ca;
		values[7] = sh * sa * cb + ch * sb;
		values[8] = -sh * sa * sb + ch * cb;

		mat.set(values);
	}

	/**
	 * Set rotation from euler angles.
	 * 
	 * @param vec
	 *            A vector containing the angles for heading, attitude and bank.
	 * @param angleUnit
	 *            Unit of the rotations
	 * @see #setEuler(double, double, double, AngleUnit)
	 */
	public void setEuler(Vector3d vec, AngleUnit angleUnit) {
		setEuler(vec.getX(), vec.getY(), vec.getZ(), angleUnit);
	}

	/**
	 * Set rotation from matrix.
	 * 
	 * @param mat
	 *            The rotation matrix.
	 */
	public void setMatrix(Matrix3d mat) {
		this.mat = mat;
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return BASE_TAG;
	}

	/**
	 * Constructs a {@link Rotation3D} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link Rotation3D} object containing all the information
	 *         given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element objectElement, Rotation3D type)
			throws ParsingException {

		try {

			Element m = objectElement.getFirstChildElement(MATRIX_TAG);
			Elements values = m.getChildElements(VALUE_TAG);

			if (values.size() != 9) {
				throw new ParsingException("Parsing Error: Matrix "
						+ "must have exactly nine entries.");
			}

			Matrix3d mat = new Matrix3d();
			mat.setM00(Double.parseDouble(values.get(0).getValue()));
			mat.setM01(Double.parseDouble(values.get(1).getValue()));
			mat.setM02(Double.parseDouble(values.get(2).getValue()));
			mat.setM10(Double.parseDouble(values.get(3).getValue()));
			mat.setM11(Double.parseDouble(values.get(4).getValue()));
			mat.setM12(Double.parseDouble(values.get(5).getValue()));
			mat.setM20(Double.parseDouble(values.get(6).getValue()));
			mat.setM21(Double.parseDouble(values.get(7).getValue()));
			mat.setM22(Double.parseDouble(values.get(8).getValue()));

			type.setMatrix(mat);

		} catch (NullPointerException ex) {

			// this happens when an element or attribute that is required is
			// not present
			throw new ParsingException("Missing element or attribute "
					+ "in document.", ex);
		} catch (NumberFormatException e) {
			throw new ParsingException("could not parse matrix values");
		}
	}

	/**
	 * Serializes the {@link Rotation3D} object into a given XOM {@link Element}
	 * .
	 * 
	 * @param parent
	 *            The {@link Element} to serialize the object into. The given
	 *            {@link Element} object should have the base tag defined by
	 *            this class. (see {@link #getBaseTag()})
	 * @see #getBaseTag()
	 */
	public void fillInto(Element parent) {

		Elements old = parent.getChildElements(MATRIX_TAG);
		for (int i = 0; i < old.size(); i++) {
			parent.removeChild(old.get(i));
		}

		Element m = new Element(MATRIX_TAG);

		Element m00 = new Element(VALUE_TAG);
		m00.appendChild(String.valueOf(mat.m00));
		m.appendChild(m00);

		Element m01 = new Element(VALUE_TAG);
		m01.appendChild(String.valueOf(mat.m01));
		m.appendChild(m01);

		Element m02 = new Element(VALUE_TAG);
		m02.appendChild(String.valueOf(mat.m02));
		m.appendChild(m02);

		Element m10 = new Element(VALUE_TAG);
		m10.appendChild(String.valueOf(mat.m10));
		m.appendChild(m10);

		Element m11 = new Element(VALUE_TAG);
		m11.appendChild(String.valueOf(mat.m11));
		m.appendChild(m11);

		Element m12 = new Element(VALUE_TAG);
		m12.appendChild(String.valueOf(mat.m12));
		m.appendChild(m12);

		Element m20 = new Element(VALUE_TAG);
		m20.appendChild(String.valueOf(mat.m20));
		m.appendChild(m20);

		Element m21 = new Element(VALUE_TAG);
		m21.appendChild(String.valueOf(mat.m21));
		m.appendChild(m21);

		Element m22 = new Element(VALUE_TAG);
		m22.appendChild(String.valueOf(mat.m22));
		m.appendChild(m22);

		parent.appendChild(m);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String info = "[" + getBaseTag();
		info += " matrix=" + mat.m00 + ", " + mat.m01 + ", " + mat.m02;
		info += ", " + mat.m10 + ", " + mat.m11 + ", " + mat.m12;
		info += ", " + mat.m20 + ", " + mat.m21 + ", " + mat.m22;
		info += "]";
		return info;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {
		try {
			if (!(obj instanceof Rotation3D))
				return false;

			Rotation3D other = (Rotation3D) obj;

			return other.mat.equals(mat);
		} catch (Exception e) {
			return false;
		}
	}
}
