/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import de.unibi.airobots.btl.common.ElementParser;

/**
 * The Person class holds the informations about one Person, who BIRON detects.
 * This class is the object, the PersonSensor return. The informations are:
 * <ul>
 * <li>the angle
 * <li>the distance
 * <li>if the Person is facing
 * <li>the gazing
 * <li>if the Person is the current person-of-interest
 * <li>if the Person is talking
 * <li>if the Person is walking
 * <li>the Person's ID
 * <li>the Person's name, if known to BIRON
 * <li>the distance to a neighbour, if present
 * <li>the neighbour's ID, if present
 * <li>two timestamps (inserted, updated)
 * </ul>
 * 
 * @author tboll
 * 
 */
public class PersonData extends Type {

	private double angle;
	private double distance;
	private boolean facing;
	private double gazing;
	private boolean poi;
	private boolean talking;
	private boolean walking;
	private int id;
	private String name;
	private double neighbourDistance;
	private int neighbourId;
	private PositionData positionData = new PositionData(0, 0, 0, 0);

	public PersonData() {

	}

	public void setAngle(double angle) {
		this.angle = angle;
	}

	public void setDistance(double distance) {
		this.distance = distance;
	}

	public void setFacing(boolean facing) {
		this.facing = facing;
	}

	public void setGazing(double gazing) {
		this.gazing = gazing;
	}

	public void setPoi(boolean poi) {
		this.poi = poi;
	}

	public void setTalking(boolean talking) {
		this.talking = talking;
	}

	public void setWalking(boolean walking) {
		this.walking = walking;
	}

	public void setId(int id) {
		this.id = id;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setNeighbourDistance(double neighbourDistance) {
		this.neighbourDistance = neighbourDistance;
	}

	public void setNeighbourId(int neighbourId) {
		this.neighbourId = neighbourId;
	}

	/**
	 * A getter for the angle, the Person is standing.
	 * 
	 * @return the angle the Person is standing.
	 */
	public double getAngle() {
		return angle;
	}

	/**
	 * A getter for the Person's distance to BIRON.
	 * 
	 * @return the Person's distance to BIRON in millimeters.
	 */
	public double getDistance() {
		return distance;
	}

	/**
	 * A getter to check, if the Person is facing BIRON or not.
	 * 
	 * @return whether the Person is facing BIRON.
	 */
	public boolean isFacing() {
		return facing;
	}

	/**
	 * A getter to check, if the Person is gazing and how much.
	 * 
	 * @return whether the Person is gazing.
	 */
	public double getGazing() {
		return gazing;
	}

	/**
	 * A getter to check, if the Person is BIRON's current person-of-interest.
	 * 
	 * @return if the Person is the person-of-interest.
	 */
	public boolean isPoi() {
		return poi;
	}

	/**
	 * A getter to check, if the Person is currently talking or not.
	 * 
	 * @return whether the Person is talking.
	 */
	public boolean isTalking() {
		return talking;
	}

	/**
	 * A getter to check, if the Person is currently
	 * 
	 * @return whether the Person is walking.
	 */
	public boolean isWalking() {
		return walking;
	}

	/**
	 * A getter for the Person's internal ID.
	 * 
	 * @return the Person's internal ID.
	 */
	public int getId() {
		return id;
	}

	/**
	 * A getter for the Person's name. If the Person is known to BIRON, a name
	 * will be assigned.
	 * 
	 * @return the Person's name, if known.
	 */
	public String getName() {
		return name;
	}

	/**
	 * A getter for a neighbouring Person's distance. If another Person is a
	 * neighbour of this Person, the distance to this Person will be returned.
	 * 
	 * @return the distance to a neighbour.
	 */
	public double getNeighbourDistance() {
		return neighbourDistance;
	}

	/**
	 * A getter for the neighbouring Person's ID. If another Person is a
	 * neighbour of this Person, the ID of this Person will be returned.
	 * 
	 * @return the internal ID of the neighbour.
	 */
	public int getNeighbourId() {
		return neighbourId;
	}

	/**
	 * A getter for the global position of a person.
	 * 
	 * 
	 */
	public PositionData getPositionData() {
		return positionData;
	}

	/**
	 * A setter for the global position of a person. This is the postion where
	 * the robot has seen this person last * @param positionData global position
	 * of a person
	 */
	public void setPositionData(PositionData positionData) {
		this.positionData = positionData;
	}

	/**
	 * @return a String with the Person's data.
	 */
	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("#PERSON# ");
		strb.append("id: " + id + "; ");
		strb.append("timestamp: " + getTimestamp() + "; ");
		strb.append("name: \"" + name + "\"; ");
		strb.append("angle: " + angle + "; ");
		strb.append("distance: " + distance + "; ");
		strb.append("facing: " + facing + "; ");
		strb.append("gazing: " + gazing + "; ");
		strb.append("poi: " + poi + "; ");
		strb.append("talking: " + talking + "; ");
		strb.append("walking: " + walking + "; ");
		strb.append("nb_dist: " + neighbourDistance + "; ");
		strb.append("nb_id: " + neighbourId + ";");
		return strb.toString();
	}

	public static String getBaseTag() {
		return "PERSON";
	}

	public static void fromElement(Element element, PersonData type) {

		Type.fromElement(element, type);

		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals("STATES")) {
					double tmpAngle = ElementParser.getDoubleAttributeValue(
							elem, "ang");
					type.setAngle(tmpAngle * Math.PI / 180.0);
					type.setDistance(ElementParser.getDoubleAttributeValue(
							elem, "dist"));
					type.setFacing(ElementParser.getBooleanAttributeValue(elem,
							"facing"));
					type.setGazing(ElementParser.getDoubleAttributeValue(elem,
							"gazing") * Math.PI / 180.0);
					type.setPoi(ElementParser.getBooleanAttributeValue(elem,
							"poi"));
					type.setTalking(ElementParser.getBooleanAttributeValue(
							elem, "talking"));
					type.setWalking(ElementParser.getBooleanAttributeValue(
							elem, "walking"));
				} else if (elem.getLocalName().equals("IDENT")) {
					type.setId(ElementParser.getIntAttributeValue(elem, "id"));
					type.setName(ElementParser.getAttributeValue(elem, "name"));
				} else if (elem.getLocalName().equals("NEIGHBOUR")) {
					type.setNeighbourDistance(ElementParser
							.getDoubleAttributeValue(elem, "dist"));
					type.setNeighbourId(ElementParser.getIntAttributeValue(
							elem, "id"));
				}
			}
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element statesElement = new Element("STATES");
		parent.appendChild(statesElement);
		statesElement.addAttribute(new Attribute("ang", String
				.valueOf(getAngle() * 180.0 / Math.PI)));
		statesElement.addAttribute(new Attribute("dist", String
				.valueOf(getDistance())));
		statesElement.addAttribute(new Attribute("facing", (isFacing() ? "YES"
				: "NO")));
		statesElement.addAttribute(new Attribute("gazing", String
				.valueOf(getGazing() * 180.0 / Math.PI)));
		statesElement.addAttribute(new Attribute("poi",
				(isPoi() ? "YES" : "NO")));
		statesElement.addAttribute(new Attribute("talking",
				(isTalking() ? "YES" : "NO")));
		statesElement.addAttribute(new Attribute("walking",
				(isWalking() ? "YES" : "NO")));

		Element identElement = new Element("IDENT");
		parent.appendChild(identElement);
		identElement.addAttribute(new Attribute("id", String.valueOf(getId())));
		identElement.addAttribute(new Attribute("name", getName()));

		Element neighbourElement = new Element("NEIGHBOUR");
		parent.appendChild(neighbourElement);
		neighbourElement.addAttribute(new Attribute("dist", String
				.valueOf(getNeighbourDistance())));
		neighbourElement.addAttribute(new Attribute("id", String
				.valueOf(getNeighbourId())));
	}
}
