/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.units;

/**
 * This helper class converts any value from any unit to any other unit.
 * 
 * @author lziegler
 */
public class UnitConverter {

	/**
	 * This exception indicates, that a conversion can not be done, because one
	 * of the given units is not known to the converter.
	 * 
	 * @author lziegler
	 */
	public static class UnknownUnitException extends RuntimeException {
		private static final long serialVersionUID = 2817908982356365437L;

		public UnknownUnitException(String message) {
			super(message);
		}
	}

	/**
	 * Converts an integer value from any length unit to any other length unit.
	 * The return value will be rounded, if the target unit is more general than
	 * the source unit.
	 * 
	 * @param value
	 *            The integer value to convert.
	 * @param from
	 *            The unit of the given value.
	 * @param to
	 *            The desired unit.
	 * @return An integer value, that corresponds to the given value, but in the
	 *         desired length unit. This value will be rounded, if the desired
	 *         unit is a more general one.
	 * @throws UnknownUnitException
	 *             Is thrown if the converter does not know one of the given
	 *             units.
	 */
	public static int convert(int value, LengthUnit from, LengthUnit to)
			throws UnknownUnitException {
		float asMillimeter = convertFrom(value, from);
		return Math.round(convertTo(asMillimeter, to));
	}

	/**
	 * Converts an long value from any length unit to any other length unit. The
	 * return value will be rounded, if the target unit is more general than the
	 * source unit.
	 * 
	 * @param value
	 *            The long value to convert.
	 * @param from
	 *            The unit of the given value.
	 * @param to
	 *            The desired unit.
	 * @return An long value, that corresponds to the given value, but in the
	 *         desired length unit. This value will be rounded, if the desired
	 *         unit is a more general one.
	 * @throws UnknownUnitException
	 *             Is thrown if the converter does not know one of the given
	 *             units.
	 */
	public static long convert(long value, LengthUnit from, LengthUnit to)
			throws UnknownUnitException {
		double asMillimeter = convertFrom(value, from);
		return Math.round(convertTo(asMillimeter, to));
	}

	/**
	 * Converts an double value from any length unit to any other length unit;
	 * 
	 * @param value
	 *            The double value to convert.
	 * @param from
	 *            The unit of the given value.
	 * @param to
	 *            The desired unit.
	 * @return An double value, that corresponds to the given value, but in the
	 *         desired length unit.
	 * @throws UnknownUnitException
	 *             Is thrown if the converter does not know one of the given
	 *             units.
	 */
	public static double convert(double value, LengthUnit from, LengthUnit to)
			throws UnknownUnitException {
		double asMillimeter = convertFrom(value, from);
		return convertTo(asMillimeter, to);
	}

	/**
	 * Converts an float value from any length unit to any other length unit;
	 * 
	 * @param value
	 *            The float value to convert.
	 * @param from
	 *            The unit of the given value.
	 * @param to
	 *            The desired unit.
	 * @return An float value, that corresponds to the given value, but in the
	 *         desired length unit.
	 * @throws UnknownUnitException
	 *             Is thrown if the converter does not know one of the given
	 *             units.
	 */
	public static float convert(float value, LengthUnit from, LengthUnit to)
			throws UnknownUnitException {
		float asMillimeter = convertFrom(value, from);
		return convertTo(asMillimeter, to);
	}

	/**
	 * Converts an double value from any angle unit to any other angle unit;
	 * 
	 * @param value
	 *            The double value to convert.
	 * @param from
	 *            The unit of the given value.
	 * @param to
	 *            The desired unit.
	 * @return An double value, that corresponds to the given value, but in the
	 *         desired length unit.
	 * @throws UnknownUnitException
	 *             Is thrown if the converter does not know one of the given
	 *             units.
	 */
	public static double convert(double value, AngleUnit from, AngleUnit to)
			throws UnknownUnitException {
		double asRadiant = convertFrom(value, from);
		return convertTo(asRadiant, to);
	}

	/**
	 * Converts an float value from any angle unit to any other angle unit;
	 * 
	 * @param value
	 *            The float value to convert.
	 * @param from
	 *            The unit of the given value.
	 * @param to
	 *            The desired unit.
	 * @return An float value, that corresponds to the given value, but in the
	 *         desired length unit.
	 * @throws UnknownUnitException
	 *             Is thrown if the converter does not know one of the given
	 *             units.
	 */
	public static float convert(float value, AngleUnit from, AngleUnit to)
			throws UnknownUnitException {
		float asRadiant = convertFrom(value, from);
		return convertTo(asRadiant, to);
	}

	/**
	 * Converts an long value from any time unit to any other time unit;
	 * 
	 * @param value
	 *            The long value to convert.
	 * @param from
	 *            The unit of the given value.
	 * @param to
	 *            The desired unit.
	 * @return An long value, that corresponds to the given value, but in the
	 *         desired time unit.
	 * @throws UnknownUnitException
	 *             Is thrown if the converter does not know one of the given
	 *             units.
	 */
	public static long convert(long value, TimeUnit from, TimeUnit to)
			throws UnknownUnitException {
		long asMicroSeconds = convertFrom(value, from);
		return convertTo(asMicroSeconds, to);
	}

	private static double convertFrom(double value, LengthUnit unit)
			throws UnknownUnitException {
		switch (unit) {
		case METER:
			return value * 1000d;
		case CENTIMETER:
			return value * 10d;
		case MILLIMETER:
			return value;
		default:
			throw new UnknownUnitException("Don't know how to convert from "
					+ unit.name());
		}
	}

	private static double convertTo(double value, LengthUnit unit)
			throws UnknownUnitException {
		switch (unit) {
		case METER:
			return value / 1000d;
		case CENTIMETER:
			return value / 10d;
		case MILLIMETER:
			return value;
		default:
			throw new UnknownUnitException("Don't know how to convert to "
					+ unit.name());
		}
	}

	private static float convertFrom(float value, LengthUnit unit)
			throws UnknownUnitException {
		switch (unit) {
		case METER:
			return value * 1000f;
		case CENTIMETER:
			return value * 10f;
		case MILLIMETER:
			return value;
		default:
			throw new UnknownUnitException("Don't know how to convert from "
					+ unit.name());
		}
	}

	private static float convertTo(float value, LengthUnit unit)
			throws UnknownUnitException {
		switch (unit) {
		case METER:
			return value / 1000f;
		case CENTIMETER:
			return value / 10f;
		case MILLIMETER:
			return value;
		default:
			throw new UnknownUnitException("Don't know how to convert to "
					+ unit.name());
		}
	}

	private static double convertFrom(double value, AngleUnit unit)
			throws UnknownUnitException {
		switch (unit) {
		case DEGREE:
			return value / 180.0 * Math.PI;
		case RADIAN:
			return value;
		default:
			throw new UnknownUnitException("Don't know how to convert from "
					+ unit.name());
		}
	}

	private static double convertTo(double value, AngleUnit unit)
			throws UnknownUnitException {
		switch (unit) {
		case DEGREE:
			return value / Math.PI * 180.0;
		case RADIAN:
			return value;
		default:
			throw new UnknownUnitException("Don't know how to convert to "
					+ unit.name());
		}
	}

	private static float convertFrom(float value, AngleUnit unit)
			throws UnknownUnitException {
		switch (unit) {
		case DEGREE:
			return value / 180f * (float) Math.PI;
		case RADIAN:
			return value;
		default:
			throw new UnknownUnitException("Don't know how to convert from "
					+ unit.name());
		}
	}

	private static float convertTo(float value, AngleUnit unit)
			throws UnknownUnitException {
		switch (unit) {
		case DEGREE:
			return value / (float) Math.PI * 180f;
		case RADIAN:
			return value;
		default:
			throw new UnknownUnitException("Don't know how to convert to "
					+ unit.name());
		}
	}

	private static long convertTo(long value, TimeUnit unit)
			throws UnknownUnitException {
		switch (unit) {
		case MICROSECONDS:
			return value;
		case MILLISECONDS:
			return value / 1000L;
		case SECONDS:
			return convertTo(value, TimeUnit.MILLISECONDS) / 1000L;
		case MINUTES:
			return convertTo(value, TimeUnit.SECONDS) / 60L;
		case HOURS:
			return convertTo(value, TimeUnit.MINUTES) / 60L;
		case DAYS:
			return convertTo(value, TimeUnit.HOURS) / 24L;
		default:
			throw new UnknownUnitException("Don't know how to convert to "
					+ unit.name());
		}
	}

	private static long convertFrom(long value, TimeUnit unit)
			throws UnknownUnitException {
		switch (unit) {
		case MICROSECONDS:
			return value;
		case MILLISECONDS:
			return value * 1000L;
		case SECONDS:
			return convertFrom(value, TimeUnit.MILLISECONDS) * 1000L;
		case MINUTES:
			return convertFrom(value, TimeUnit.SECONDS) * 60L;
		case HOURS:
			return convertFrom(value, TimeUnit.MINUTES) * 60L;
		case DAYS:
			return convertFrom(value, TimeUnit.HOURS) * 24L;
		default:
			throw new UnknownUnitException("Don't know how to convert to "
					+ unit.name());
		}
	}
}
